import sys
from _typeshed import Unused
from datetime import datetime, timedelta, tzinfo
from typing import Any, ClassVar, Literal, Protocol, TypeVar, type_check_only
from typing_extensions import Self

from ..relativedelta import relativedelta
from ._common import _tzinfo, enfold as enfold, tzrangebase

if sys.platform == "win32":
    from .win import tzwin as tzwin, tzwinlocal as tzwinlocal
else:
    tzwin: None
    tzwinlocal: None

_DateTimeT = TypeVar("_DateTimeT", bound=datetime)

ZERO: timedelta
EPOCH: datetime
EPOCHORDINAL: int

class tzutc(tzinfo):
    def utcoffset(self, dt: datetime | None) -> timedelta | None: ...
    def dst(self, dt: datetime | None) -> timedelta | None: ...
    def tzname(self, dt: datetime | None) -> str: ...
    def is_ambiguous(self, dt: datetime | None) -> bool: ...
    def fromutc(self, dt: _DateTimeT) -> _DateTimeT: ...
    def __eq__(self, other: object) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other: object) -> bool: ...
    __reduce__ = object.__reduce__

UTC: tzutc

class tzoffset(tzinfo):
    def __init__(self, name: str | None, offset: float | timedelta) -> None: ...
    def utcoffset(self, dt: datetime | None) -> timedelta | None: ...
    def dst(self, dt: datetime | None) -> timedelta | None: ...
    def is_ambiguous(self, dt: datetime | None) -> bool: ...
    def tzname(self, dt: datetime | None) -> str: ...
    def fromutc(self, dt: _DateTimeT) -> _DateTimeT: ...
    def __eq__(self, other: object) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other: object) -> bool: ...
    __reduce__ = object.__reduce__
    @classmethod
    def instance(cls, name: str | None, offset: float | timedelta) -> tzoffset: ...

class tzlocal(_tzinfo):
    def __init__(self) -> None: ...
    def utcoffset(self, dt: datetime | None) -> timedelta | None: ...
    def dst(self, dt: datetime | None) -> timedelta | None: ...
    def tzname(self, dt: datetime | None) -> str: ...
    def is_ambiguous(self, dt: datetime | None) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other: object) -> bool: ...
    __reduce__ = object.__reduce__

class _ttinfo:
    __slots__ = ["offset", "delta", "isdst", "abbr", "isstd", "isgmt", "dstoffset"]
    offset: float
    delta: timedelta
    isdst: bool
    abbr: str
    isstd: bool
    isgmt: bool
    dstoffset: timedelta
    def __init__(self) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other: object) -> bool: ...

@type_check_only
class _TZFileReader(Protocol):
    # optional attribute:
    # name: str
    def read(self, size: int, /) -> bytes: ...
    def seek(self, target: int, whence: Literal[1], /) -> object: ...

class tzfile(_tzinfo):
    def __init__(self, fileobj: str | _TZFileReader, filename: str | None = None) -> None: ...
    def is_ambiguous(self, dt: datetime | None, idx: int | None = None) -> bool: ...
    def utcoffset(self, dt: datetime | None) -> timedelta | None: ...
    def dst(self, dt: datetime | None) -> timedelta | None: ...
    def tzname(self, dt: datetime | None) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def __ne__(self, other: object) -> bool: ...
    def __reduce__(self) -> tuple[type[Self], tuple[None, str], dict[str, Any]]: ...
    def __reduce_ex__(self, protocol: Unused) -> tuple[type[Self], tuple[None, str], dict[str, Any]]: ...

class tzrange(tzrangebase):
    hasdst: bool
    def __init__(
        self,
        stdabbr: str,
        stdoffset: int | timedelta | None = None,
        dstabbr: str | None = None,
        dstoffset: int | timedelta | None = None,
        start: relativedelta | None = None,
        end: relativedelta | None = None,
    ) -> None: ...
    def transitions(self, year: int) -> tuple[datetime, datetime]: ...
    def __eq__(self, other: object) -> bool: ...

class tzstr(tzrange):
    hasdst: bool
    def __init__(self, s: str, posix_offset: bool = False) -> None: ...
    @classmethod
    def instance(cls, name: str | None, offset: float | timedelta) -> tzoffset: ...

@type_check_only
class _ICalReader(Protocol):
    # optional attribute:
    # name: str
    def read(self) -> str: ...

class tzical:
    def __init__(self, fileobj: str | _ICalReader) -> None: ...
    def keys(self) -> list[str]: ...
    def get(self, tzid: str | None = None) -> tzinfo | None: ...

TZFILES: list[str]
TZPATHS: list[str]

def datetime_exists(dt: datetime, tz: tzinfo | None = None) -> bool: ...
def datetime_ambiguous(dt: datetime, tz: tzinfo | None = None) -> bool: ...
def resolve_imaginary(dt: _DateTimeT) -> _DateTimeT: ...

# Singleton type defined locally in a function. Calls itself "GettzFunc".
@type_check_only
class _GetTZ:
    def __call__(self, name: str | None = None) -> tzinfo | None: ...
    def set_cache_size(self, size: int) -> None: ...
    def cache_clear(self) -> None: ...
    def nocache(self, name: str | None) -> tzinfo | None: ...

gettz: _GetTZ



import sys
from datetime import datetime
from typing_extensions import Self

from ._common import tzrangebase
from .tz import (
    datetime_ambiguous as datetime_ambiguous,
    datetime_exists as datetime_exists,
    enfold as enfold,
    gettz as gettz,
    resolve_imaginary as resolve_imaginary,
    tzfile as tzfile,
    tzical as tzical,
    tzlocal as tzlocal,
    tzoffset as tzoffset,
    tzrange as tzrange,
    tzstr as tzstr,
    tzutc as tzutc,
)

# UTC, tzwin, tzwinlocal are defined in this class
# otherwise pyright complains about unknown import symbol:
if sys.platform == "win32":
    class tzwinbase(tzrangebase):
        hasdst: bool
        def __eq__(self, other: tzwinbase) -> bool: ...  # type: ignore[override]
        @staticmethod
        def list() -> list[str]: ...
        def display(self) -> str | None: ...
        def transitions(self, year: int) -> tuple[datetime, datetime] | None: ...

    class tzwin(tzwinbase):
        hasdst: bool
        def __init__(self, name: str) -> None: ...
        def __reduce__(self) -> tuple[type[Self], tuple[str, ...]]: ...  # type: ignore[override]

    class tzwinlocal(tzwinbase):
        hasdst: bool
        def __init__(self) -> None: ...
        def __reduce__(self) -> tuple[type[Self], tuple[str, ...]]: ...  # type: ignore[override]

else:
    tzwin: None
    tzwinlocal: None

UTC: tzutc

__all__ = [
    "tzutc",
    "tzoffset",
    "tzlocal",
    "tzfile",
    "tzrange",
    "tzstr",
    "tzical",
    "tzwin",
    "tzwinlocal",
    "gettz",
    "enfold",
    "datetime_ambiguous",
    "datetime_exists",
    "resolve_imaginary",
    "UTC",
    "DeprecatedTzFormatWarning",
]

class DeprecatedTzFormatWarning(Warning): ...



from _typeshed import Incomplete
from collections.abc import Callable
from io import BytesIO
from tarfile import _Fileobj
from typing import Final, TypeAlias, TypeVar, overload
from typing_extensions import Self, deprecated

from dateutil.tz import tzfile as _tzfile

_T = TypeVar("_T")
_MetadataType: TypeAlias = dict[str, Incomplete]

__all__ = ["get_zonefile_instance", "gettz", "gettz_db_metadata"]

ZONEFILENAME: Final[str]
METADATA_FN: Final[str]

class tzfile(_tzfile):
    # source code does this override, changing the type
    def __reduce__(self) -> tuple[Callable[[str], Self], tuple[str]]: ...  # type: ignore[override]

def getzoneinfofile_stream() -> BytesIO | None: ...

class ZoneInfoFile:
    zones: dict[str, _tzfile]
    metadata: _MetadataType | None
    def __init__(self, zonefile_stream: _Fileobj | None = None) -> None: ...
    @overload
    def get(self, name: str, default: None = None) -> _tzfile | None: ...
    @overload
    def get(self, name: str, default: _tzfile) -> _tzfile: ...
    @overload
    def get(self, name: str, default: _T) -> _tzfile | _T: ...

def get_zonefile_instance(new_instance: bool = False) -> ZoneInfoFile: ...
@deprecated(
    "zoneinfo.gettz() will be removed in future versions, to use the dateutil-provided "
    "zoneinfo files, instantiate a ZoneInfoFile object and use ZoneInfoFile.zones.get() instead."
)
def gettz(name: str) -> _tzfile: ...
@deprecated(
    "zoneinfo.gettz_db_metadata() will be removed in future versions, to use the "
    "dateutil-provided zoneinfo files, ZoneInfoFile object and query the 'metadata' attribute instead."
)
def gettz_db_metadata() -> _MetadataType: ...



from datetime import date
from typing import Final, Literal

__all__ = ["easter", "EASTER_JULIAN", "EASTER_ORTHODOX", "EASTER_WESTERN"]

EASTER_JULIAN: Final = 1
EASTER_ORTHODOX: Final = 2
EASTER_WESTERN: Final = 3

def easter(year: int, method: Literal[1, 2, 3] = 3) -> date: ...



from _typeshed import SupportsRead
from datetime import date, datetime, time, tzinfo
from typing import TypeAlias

_Readable: TypeAlias = SupportsRead[str | bytes]
_TakesAscii: TypeAlias = str | bytes | _Readable

__all__ = ["isoparse", "isoparser"]

class isoparser:
    def __init__(self, sep: str | bytes | None = None) -> None: ...
    def isoparse(self, dt_str: _TakesAscii) -> datetime: ...
    def parse_isodate(self, datestr: _TakesAscii) -> date: ...
    def parse_isotime(self, timestr: _TakesAscii) -> time: ...
    def parse_tzstr(self, tzstr: _TakesAscii, zero_as_utc: bool = True) -> tzinfo: ...

def isoparse(dt_str: _TakesAscii) -> datetime: ...
